extension OpenDatetimeExtensions on DateTime {
  /// Returns the current UTC time as an ISO 8601 string.
  String nowString() => DateTime.now().toUtc().toIso8601String();

  /// Checks if the date is today.
  bool get isToday => _isSameDay(this, DateTime.now());

  /// Checks if the date is yesterday.
  bool get isYesterday =>
      _isSameDay(this, DateTime.now().subtract(const Duration(days: 1)));

  /// Checks if the date is tomorrow.
  bool get isTomorrow =>
      _isSameDay(this, DateTime.now().add(const Duration(days: 1)));

  /// Time of day checks.
  bool get isMorning => hour >= 4 && hour < 12;
  bool get isAfternoon => hour >= 12 && hour < 18;
  bool get isEvening => hour < 4 || hour >= 18;

  /// Returns the time of day as a string.
  String get timeOfDay {
    if (isMorning) return 'morning';
    if (isAfternoon) return 'afternoon';
    return 'evening';
  }

  /// Returns the first date of the current week.
  DateTime get firstDateOfTheWeek => subtract(Duration(days: weekday - 1));

  /// Returns the last date of the current week.
  DateTime get lastDateOfTheWeek =>
      add(Duration(days: DateTime.daysPerWeek - weekday));

  /// Formats a duration (in seconds) into "HH:mm:ss".
  String formatSeconds(int value) {
    final hours = (value ~/ 3600).toString().padLeft(2, '0');
    final minutes = ((value % 3600) ~/ 60).toString().padLeft(2, '0');
    final seconds = (value % 60).toString().padLeft(2, '0');
    return "$hours:$minutes:$seconds";
  }

  /// Formats a duration (in minutes) into "HH:mm".
  /// If `allowZero` is false, replaces "HH:00" with "HH:01".
  String formatMinutes(int value, {bool allowZero = false}) {
    final hours = (value ~/ 60).toString().padLeft(2, '0');
    final minutes = (value % 60).toString().padLeft(2, '0');
    return minutes == '00' && !allowZero ? "$hours:01" : "$hours:$minutes";
  }

  /// Returns the last day of the current month.
  DateTime get lastDayOfMonth =>
      (month < 12) ? DateTime(year, month + 1, 0) : DateTime(year + 1, 1, 0);

  /// Private helper method to check if two dates fall on the same calendar day.
  static bool _isSameDay(DateTime date1, DateTime date2) {
    return date1.year == date2.year &&
        date1.month == date2.month &&
        date1.day == date2.day;
  }
}
