extension OpenStringExtension on String {
  /// Removes all occurrences of [rhs] from the string.
  String operator -(String rhs) => replaceAll(rhs, '');

  /// Truncates the string to [maxLength] and appends '...' if truncated.
  String truncateTo(int maxLength) {
    if (maxLength <= 0) return '';
    return length <= maxLength ? this : '${substring(0, maxLength - 3)}...';
  }

  /// Shortens the string to [maxLength] characters, preserving whole words.
  String shortenByWords(int maxLength) =>
      shortenByDelimiterWithCount(maxLength);

  /// Shortens the string to [maxLength] characters using [delimiter] as a word boundary.
  String shortenByDelimiterWithCount(int maxLength, [String delimiter = ' ']) {
    if (maxLength <= 0) return '';
    if (length <= maxLength) return this;

    final cutIndex = substring(0, maxLength).lastIndexOf(delimiter);
    if (cutIndex == -1) return substring(0, maxLength);

    final shortened = substring(0, cutIndex).trim();
    final remainingWordsCount =
        split(delimiter).length - shortened.split(delimiter).length;

    return remainingWordsCount > 0
        ? '$shortened +$remainingWordsCount'
        : shortened;
  }

  /// Splits the string into chunks of length [len], padding with [filler] if necessary.
  Iterable<String> splitByLength(int len, {String filler = '0'}) sync* {
    if (len <= 0)
      throw ArgumentError.value(len, 'len', 'Must be greater than 0');
    final paddedString = padRight((length + len - 1) ~/ len * len, filler);
    for (var i = 0; i < paddedString.length; i += len) {
      yield paddedString.substring(i, i + len);
    }
  }

  /// Capitalizes the first character of the string.
  String capitalize() =>
      isNotEmpty ? '${this[0].toUpperCase()}${substring(1)}' : this;

  /// Decapitalizes the first character of the string.
  String decapitalize() =>
      isNotEmpty ? '${this[0].toLowerCase()}${substring(1)}' : this;
}
